/*
Google Music Frame :: Application

This file contains main application class and its wrapper for DBus.

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#if UNITY_QUICK_LIST

using Fenryxo.Logger;
using GMF;
using Unity;
namespace GMF.Helpers{
	
	/**
	 * Manages dock item at Unity Launcher
	 */
	public class UnityLauncher: BaseHelper{
		private LauncherEntry dock_item;
		Dbusmenu.Menuitem pp_item;
	
		public UnityLauncher(GMF.Application app, GMF.Player? player){
			base(app, player);
		}
		
		~ UnityLauncher(){
			this.remove_menu();
		}
		
		public void start(){
			log_debug("Unity Launcher Entry for %s.desktop", this.app.desktop_entry);
			this.dock_item = LauncherEntry.get_for_desktop_id(
				this.app.desktop_entry + ".desktop");
			this.create_menu();
			this.app.terminated.connect(this.remove_menu);
		}
		
		
		
		protected override void create_menu(){
			log_debug("Creating Unity Launcher Quick List");
			Dbusmenu.Menuitem item;
			var menu = new Dbusmenu.Menuitem ();
			
			Actions skip_action = this.player.playback_state == "playing"
				? Actions.PLAY : Actions.PAUSE;

//~ Weird build errors: https://launchpadlibrarian.net/83998439/buildlog_ubuntu-oneiric-i386.google-music-frame_0.2.2%2Bnext-0ubuntu0~bzr76~16~oneiric1_FAILEDTOBUILD.txt.gz			
//~ 			var actions = Actions.all();
//~ 			foreach(var action in actions){
			for(int a = 0; a < Actions.COUNT; a++){
				// Needed: If "a" is passed directly to this.trigger_action(),
				// it will be always 7 (the last iteration)
				Actions action = (Actions) a; 
				if(action == skip_action) continue;
				item = new Dbusmenu.Menuitem ();
				item.property_set(Dbusmenu.MENUITEM_PROP_LABEL, action.get_label());
				item.property_set(Dbusmenu.MENUITEM_PROP_ICON_NAME, action.get_icon());
				item.item_activated.connect(() => { this.trigger_action(action); });
				if(action == Actions.PAUSE || action == Actions.PLAY) this.pp_item = item;
				menu.child_append(item);
			}
			
			this.dock_item.quicklist = menu;
			
		}
		
		protected override void update_menu(){
			if(this.player.playback_state == "playing"){
				this.pp_item.property_set(Dbusmenu.MENUITEM_PROP_LABEL, Actions.PAUSE.get_label());				
				this.pp_item.property_set(Dbusmenu.MENUITEM_PROP_ICON_NAME, Actions.PAUSE.get_icon());
			}
			else{
				this.pp_item.property_set(Dbusmenu.MENUITEM_PROP_LABEL, Actions.PLAY.get_label());				
				this.pp_item.property_set(Dbusmenu.MENUITEM_PROP_ICON_NAME, Actions.PLAY.get_icon());
			}
		}
		
		protected override void remove_menu(){
			if(this.dock_item != null) this.dock_item.quicklist = null;
		}
	}
}
#endif
